@@ -31,33 +31,22 @@
  * all the necessary logic for dragging actors.
  *
  * The simplest usage of #ClutterDragAction consists in adding it to
- * a #ClutterActor and connecting to the #ClutterDragAction::drag-motion
- * signal handler to move the actor; for instance:
+ * a #ClutterActor and setting it as reactive; for instance, the following
+ * code:
  *
  * |[
- *   ClutterAction *action = clutter_drag_action_new ();
- *
- *   g_signal_connect (action, "drag-motion",
- *                     G_CALLBACK (on_drag_motion),
- *                     NULL);
- *
- *   clutter_actor_add_action (actor, action);
+ *   clutter_actor_add_action (actor, clutter_drag_action_new ());
+ *   clutter_actor_set_reactive (actor, TRUE);
  * ]|
  *
- * Where the on_drag_motion() signal handler calls clutter_actor_move_by()
- * using the delta between motion events passed to the handler:
+ * will automatically result in the actor moving to follow the pointer
+ * whenever the pointer's button is pressed over the actor and moved
+ * across the stage.
  *
- * |[
- *   static void
- *   on_drag_motion (ClutterDragAction   *action,
- *                   ClutterActor        *actor,
- *                   gfloat               delta_x,
- *                   gfloat               delta_y,
- *                   ClutterModifierType  modifiers)
- *   {
- *     clutter_actor_move_by (actor, delta_x, delta_y);
- *   }
- * ]|
+ * The #ClutterDragAction will signal the begin and the end of a dragging
+ * through the #ClutterDragAction::drag-begin and #ClutterDragAction::drag-end
+ * signals, respectively. Each pointer motion during a drag will also result
+ * in the #ClutterDragAction::drag-motion signal to be emitted.
  *
  * It is also possible to set another #ClutterActor as the dragged actor
  * by calling clutter_drag_action_set_drag_handle() from within a handle
@@ -223,7 +212,7 @@ emit_drag_end (ClutterDragAction *action,
     g_signal_emit (action, drag_signals[DRAG_END], 0,
                    actor,
                    priv->last_motion_x, priv->last_motion_y,
-                   clutter_event_get_button (event),
+                   priv->press_button,
                    clutter_event_get_state (event));
 
   /* disconnect the capture */
@@ -348,6 +337,22 @@ clutter_drag_action_set_actor (ClutterActorMeta *meta,
   CLUTTER_ACTOR_META_CLASS (clutter_drag_action_parent_class)->set_actor (meta, actor);
 }
 
+static void
+clutter_drag_action_real_drag_motion (ClutterDragAction *action,
+                                      ClutterActor      *actor,
+                                      gfloat             delta_x,
+                                      gfloat             delta_y)
+{
+  ClutterActor *drag_handle;
+
+  if (action->priv->drag_handle != NULL)
+    drag_handle = action->priv->drag_handle;
+  else
+    drag_handle = actor;
+
+  clutter_actor_move_by (drag_handle, delta_x, delta_y);
+}
+
 static void
 clutter_drag_action_set_property (GObject      *gobject,
                                   guint         prop_id,
@@ -443,6 +448,8 @@ clutter_drag_action_class_init (ClutterDragActionClass *klass)
 
   meta_class->set_actor = clutter_drag_action_set_actor;
 
+  klass->drag_motion = clutter_drag_action_real_drag_motion;
+
   /**
    * ClutterDragAction:drag-threshold:
    *
@@ -540,7 +547,6 @@ clutter_drag_action_class_init (ClutterDragActionClass *klass)
    * @delta_y: the Y component of the distance between the press event
    *   that began the dragging and the current position of the pointer,
    *   as of the latest motion event
-   * @modifiers: the modifiers of the latest motion event
    *
    * The ::drag-motion signal is emitted for each motion event after
    * the #ClutterDragAction::drag-begin signal has been emitted.
@@ -551,6 +557,13 @@ clutter_drag_action_class_init (ClutterDragActionClass *klass)
    * stage coordinates of the latest motion event you can use
    * clutter_drag_action_get_motion_coords().
    *
+   * The default handler of the signal will call clutter_actor_move_by()
+   * either on @actor or, if set, of #ClutterDragAction:drag-handle using
+   * the @delta_x and @delta_y components of the dragging motion. If you
+   * want to override the default behaviour, you can connect to this
+   * signal and call g_signal_stop_emission_by_name() from within your
+   * callback.
+   *
    * Since: 1.4
    */
   drag_signals[DRAG_MOTION] =
