@@ -256,7 +256,7 @@ _cogl_texture_quad_multiple_primitives (CoglHandle   tex_handle,
     }
 
   state.wrap_mode_overrides = NULL;
-  memset (&wrap_mode_overrides, 0, sizeof (wrap_mode_overrides));
+  memset (&state.wrap_mode_overrides, 0, sizeof (state.wrap_mode_overrides));
 
   /* We can't use hardware repeat so we need to set clamp to edge
      otherwise it might pull in edge pixels from the other side. By
@@ -516,52 +516,17 @@ _cogl_rectangles_with_multitexture_coords (
     {
       CoglHandle     layer = tmp->data;
       CoglHandle     tex_handle;
+      unsigned long  flags;
 
       if (cogl_material_layer_get_type (layer)
 	  != COGL_MATERIAL_LAYER_TYPE_TEXTURE)
 	continue;
 
       /* We need to ensure the mipmaps are ready before deciding
-       * anything else about the texture because the texture storage
-       * could completely change if it needs to be migrated out of the
-       * atlas and will affect how we validate the layer.
-       *
-       * FIXME: this needs to be generalized. There could be any
-       * number of things that might require a shuffling of the
-       * underlying texture storage. We could add two mechanisms to
-       * generalize this a bit...
-       *
-       * 1) add a _cogl_material_layer_update_storage() function that
-       * would for instance consider if mipmapping is necessary and
-       * potentially migrate the texture from an atlas.
-       *
-       * 2) allow setting of transient primitive-flags on a material
-       * that may affect the outcome of _update_storage(). One flag
-       * could indicate that we expect to sample beyond the bounds of
-       * the texture border.
-       *
-       *   flags = COGL_MATERIAL_PRIMITIVE_FLAG_VALID_BORDERS;
-       *   _cogl_material_layer_assert_primitive_flags (layer, flags)
-       *   _cogl_material_layer_update_storage (layer)
-       *   enqueue primitive in journal
-       *
-       *   when the primitive is dequeued and drawn we should:
-       *   _cogl_material_flush_gl_state (material)
-       *   draw primitive
-       *   _cogl_material_unassert_primitive_flags (layer, flags);
-       *
-       * _cogl_material_layer_update_storage should take into
-       * consideration all the asserted primitive requirements.  (E.g.
-       * there could be multiple primitives in the journal - or in a
-       * renderlist in the future - that need mipmaps or that need
-       * valid contents beyond their borders (for cogl_polygon)
-       * meaning they can't work with textures in an atas, so
-       * _cogl_material_layer_update_storage would pass on these
-       * requirements to the texture atlas backend which would make
-       * sure the referenced texture is migrated out of the atlas and
-       * mipmaps are generated.)
-       */
-      _cogl_material_layer_pre_paint (layer);
+         anything else about the texture because it could become
+         something completely different if it needs to be migrated out
+         of the atlas */
+      _cogl_material_layer_ensure_mipmaps (layer);
 
       tex_handle = cogl_material_layer_get_texture (layer);
 
@@ -618,7 +583,8 @@ _cogl_rectangles_with_multitexture_coords (
        * waste or if using GL_TEXTURE_RECTANGLE_ARB) then we don't support
        * multi texturing since we don't know if the result will end up trying
        * to texture from the waste area. */
-      if (_cogl_material_layer_has_user_matrix (layer)
+      flags = _cogl_material_layer_get_flags (layer);
+      if (flags & COGL_MATERIAL_LAYER_FLAG_HAS_USER_MATRIX
           && !_cogl_texture_can_hardware_repeat (tex_handle))
         {
           static gboolean warning_seen = FALSE;
@@ -668,7 +634,6 @@ _cogl_rectangles_with_multitexture_coords (
       /* If multitexturing failed or we are drawing with a sliced texture
        * then we only support a single layer so we pluck out the texture
        * from the first material layer... */
-      layers = cogl_material_get_layers (material);
       first_layer = layers->data;
       tex_handle = cogl_material_layer_get_texture (first_layer);
 
@@ -837,7 +802,6 @@ draw_polygon_sub_texture_cb (CoglHandle tex_handle,
   float virtual_origin_y;
   float v_to_s_scale_x;
   float v_to_s_scale_y;
-  CoglHandle source;
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
@@ -866,18 +830,12 @@ draw_polygon_sub_texture_cb (CoglHandle tex_handle,
       v += state->stride;
     }
 
-  if (G_UNLIKELY (ctx->legacy_state_set))
-    {
-      source = cogl_material_copy (ctx->source_material);
-      _cogl_material_apply_legacy_state (source);
-    }
-  else
-    source = ctx->source_material;
-
   options.flags =
+    COGL_MATERIAL_FLUSH_DISABLE_MASK |
     COGL_MATERIAL_FLUSH_LAYER0_OVERRIDE |
     COGL_MATERIAL_FLUSH_WRAP_MODE_OVERRIDES;
-
+  /* disable all except the first layer */
+  options.disable_layers = (guint32)~1;
   options.layer0_override_texture = gl_handle;
 
   /* Override the wrapping mode on all of the slices to use a
@@ -894,24 +852,9 @@ draw_polygon_sub_texture_cb (CoglHandle tex_handle,
   options.wrap_mode_overrides.values[0].t =
     COGL_MATERIAL_WRAP_MODE_OVERRIDE_CLAMP_TO_BORDER;
 
-  if (cogl_material_get_n_layers (source) != 1)
-    {
-      /* disable all except the first layer */
-      options.disable_layers = (guint32)~1;
-      options.flags |= COGL_MATERIAL_FLUSH_DISABLE_MASK;
-    }
-
-  /* If we haven't already created a derived material... */
-  if (source == ctx->source_material)
-    source = cogl_material_copy (ctx->source_material);
-  _cogl_material_apply_overrides (source, &options);
-
-  _cogl_material_flush_gl_state (source, FALSE);
+  _cogl_material_flush_gl_state (ctx->source_material, &options);
 
   GE (glDrawArrays (GL_TRIANGLE_FAN, 0, state->n_vertices));
-
-  if (G_UNLIKELY (source != ctx->source_material))
-    cogl_handle_unref (source);
 }
 
 /* handles 2d-sliced textures with > 1 slice */
@@ -984,7 +927,6 @@ _cogl_multitexture_polygon_single_primitive (const CoglTextureVertex *vertices,
   GList               *tmp;
   GLfloat             *v;
   CoglMaterialFlushOptions options;
-  CoglHandle           source;
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
@@ -1041,40 +983,18 @@ _cogl_multitexture_polygon_single_primitive (const CoglTextureVertex *vertices,
         }
     }
 
-  if (G_UNLIKELY (ctx->legacy_state_set))
-    {
-      source = cogl_material_copy (ctx->source_material);
-      _cogl_material_apply_legacy_state (source);
-    }
-  else
-    source = ctx->source_material;
-
-  options.flags = 0;
-
-  if (G_UNLIKELY (fallback_layers))
-    {
-      options.flags |= COGL_MATERIAL_FLUSH_FALLBACK_MASK;
-      options.fallback_layers = fallback_layers;
-    }
+  options.flags = COGL_MATERIAL_FLUSH_FALLBACK_MASK;
+  if (use_color)
+    options.flags |= COGL_MATERIAL_FLUSH_SKIP_GL_COLOR;
+  options.fallback_layers = fallback_layers;
   if (wrap_mode_overrides)
     {
       options.flags |= COGL_MATERIAL_FLUSH_WRAP_MODE_OVERRIDES;
       options.wrap_mode_overrides = *wrap_mode_overrides;
     }
-  if (options.flags)
-    {
-      /* If we haven't already created a derived material... */
-      if (source == ctx->source_material)
-        source = cogl_material_copy (ctx->source_material);
-      _cogl_material_apply_overrides (source, &options);
-    }
-
-  _cogl_material_flush_gl_state (source, use_color);
+  _cogl_material_flush_gl_state (ctx->source_material, &options);
 
   GE (glDrawArrays (GL_TRIANGLE_FAN, 0, n_vertices));
-
-  if (G_UNLIKELY (source != ctx->source_material))
-    cogl_handle_unref (source);
 }
 
 void
@@ -1094,8 +1014,6 @@ cogl_polygon (const CoglTextureVertex *vertices,
   GLfloat             *v;
   CoglMaterialWrapModeOverrides wrap_mode_overrides;
   CoglMaterialWrapModeOverrides *wrap_mode_overrides_p = NULL;
-  CoglHandle           original_source_material = NULL;
-  gboolean             overrode_material = FALSE;
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
@@ -1123,22 +1041,10 @@ cogl_polygon (const CoglTextureVertex *vertices,
         continue;
 
       /* Give the texture a chance to know that we're rendering
-       * non-quad shaped primitives. If the texture is in an atlas it
-       * will be migrated
-       *
-       * FIXME: this needs to be generalized. There could be any
-       * number of things that might require a shuffling of the
-       * underlying texture storage.
-       */
+         non-quad shaped primitives. If the texture is in an atlas it
+         will be migrated */
       _cogl_texture_ensure_non_quad_rendering (tex_handle);
 
-      /* We need to ensure the mipmaps are ready before deciding
-       * anything else about the texture because the texture storate
-       * could completely change if it needs to be migrated out of the
-       * atlas and will affect how we validate the layer.
-       */
-      _cogl_material_layer_pre_paint (layer);
-
       if (i == 0 && cogl_texture_is_sliced (tex_handle))
         {
 #if defined (HAVE_COGL_GLES) || defined (HAVE_COGL_GLES2)
@@ -1229,31 +1135,18 @@ cogl_polygon (const CoglTextureVertex *vertices,
 
   /* Prepare GL state */
   enable_flags = COGL_ENABLE_VERTEX_ARRAY;
+  enable_flags |= _cogl_material_get_cogl_enable_flags (ctx->source_material);
 
   if (ctx->enable_backface_culling)
     enable_flags |= COGL_ENABLE_BACKFACE_CULLING;
 
   if (use_color)
     {
-      CoglHandle override;
-      enable_flags |= COGL_ENABLE_COLOR_ARRAY;
+      enable_flags |= COGL_ENABLE_COLOR_ARRAY | COGL_ENABLE_BLEND;
       GE( glColorPointer (4, GL_UNSIGNED_BYTE,
                           stride_bytes,
                           /* NB: [X,Y,Z,TX,TY...,R,G,B,A,...] */
                           v + 3 + 2 * n_layers) );
-
-      if (!_cogl_material_get_real_blend_enabled (ctx->source_material))
-        {
-          CoglMaterialBlendEnable blend_enabled =
-            COGL_MATERIAL_BLEND_ENABLE_ENABLED;
-          original_source_material = ctx->source_material;
-          override = cogl_material_copy (original_source_material);
-          _cogl_material_set_blend_enabled (override, blend_enabled);
-
-          /* XXX: cogl_push_source () */
-          overrode_material = TRUE;
-          ctx->source_material = override;
-        }
     }
 
   _cogl_enable (enable_flags);
@@ -1289,16 +1182,6 @@ cogl_polygon (const CoglTextureVertex *vertices,
                                                  fallback_layers,
                                                  wrap_mode_overrides_p);
 
-  /* XXX: cogl_pop_source () */
-  if (overrode_material)
-    {
-      cogl_handle_unref (ctx->source_material);
-      ctx->source_material = original_source_material;
-      /* XXX: when we have weak materials then any override material
-       * should get associated with the original material so we don't
-       * create lots of one-shot materials! */
-    }
-
   /* Reset the size of the logged vertex array because rendering
      rectangles expects it to start at 0 */
   g_array_set_size (ctx->logged_vertices, 0);
