@@ -124,24 +124,25 @@ _cogl_texture_2d_foreach_sub_texture_in_region (
 }
 
 static void
-_cogl_texture_2d_set_wrap_mode_parameter (CoglTexture *tex,
-                                          GLenum wrap_mode)
+_cogl_texture_2d_set_wrap_mode_parameters (CoglTexture *tex,
+                                           GLenum wrap_mode_s,
+                                           GLenum wrap_mode_t,
+                                           GLenum wrap_mode_r)
 {
   CoglTexture2D *tex_2d = COGL_TEXTURE_2D (tex);
 
-  /* Only set the wrap mode if it's different from the current
-     value to avoid too many GL calls */
-  if (tex_2d->wrap_mode != wrap_mode)
+  /* Only set the wrap mode if it's different from the current value
+     to avoid too many GL calls. Texture 2D doesn't make use of the r
+     coordinate so we can ignore its wrap mode */
+  if (tex_2d->wrap_mode_s != wrap_mode_s ||
+      tex_2d->wrap_mode_t != wrap_mode_t)
     {
-      /* Any queued texture rectangles may be depending on the
-       * previous wrap mode... */
-      _cogl_journal_flush ();
-
       GE( glBindTexture (GL_TEXTURE_2D, tex_2d->gl_texture) );
-      GE( glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, wrap_mode) );
-      GE( glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, wrap_mode) );
+      GE( glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, wrap_mode_s) );
+      GE( glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, wrap_mode_t) );
 
-      tex_2d->wrap_mode = wrap_mode;
+      tex_2d->wrap_mode_s = wrap_mode_s;
+      tex_2d->wrap_mode_t = wrap_mode_t;
     }
 }
 
@@ -155,21 +156,8 @@ _cogl_texture_2d_free (CoglTexture2D *tex_2d)
 static gboolean
 _cogl_texture_2d_is_pot (unsigned int num)
 {
-  gboolean have_bit = FALSE;
-
   /* Make sure there is only one bit set */
-  while (num)
-    {
-      if (num & 1)
-        {
-          if (have_bit)
-            return FALSE;
-          have_bit = TRUE;
-        }
-      num >>= 1;
-    }
-
-  return TRUE;
+  return (num & (num - 1)) == 0;
 }
 
 static gboolean
@@ -231,7 +219,8 @@ _cogl_texture_2d_create_base (unsigned int     width,
   tex_2d->mag_filter = GL_LINEAR;
 
   /* Wrap mode not yet set */
-  tex_2d->wrap_mode = GL_FALSE;
+  tex_2d->wrap_mode_s = GL_FALSE;
+  tex_2d->wrap_mode_t = GL_FALSE;
 
   tex_2d->format = internal_format;
 
@@ -496,6 +485,8 @@ _cogl_texture_2d_set_region (CoglTexture    *tex,
   if (tmp_bmp_owner)
     g_free (tmp_bmp.data);
 
+  tex_2d->mipmaps_dirty = TRUE;
+
   return TRUE;
 }
 
@@ -643,7 +634,7 @@ cogl_texture_2d_vtable =
     _cogl_texture_2d_set_filters,
     _cogl_texture_2d_ensure_mipmaps,
     _cogl_texture_2d_ensure_non_quad_rendering,
-    _cogl_texture_2d_set_wrap_mode_parameter,
+    _cogl_texture_2d_set_wrap_mode_parameters,
     _cogl_texture_2d_get_format,
     _cogl_texture_2d_get_gl_format,
     _cogl_texture_2d_get_width,
