@@ -55,22 +55,11 @@ typedef enum {
   COGL_TRANSFORM_SOFTWARE_REPEAT,
 } CoglTransformResult;
 
-/* Flags given to the pre_paint method */
-typedef enum {
-  /* The texture is going to be used with filters that require
-     mipmapping. This gives the texture the opportunity to
-     automatically update the mipmap tree */
-  COGL_TEXTURE_NEEDS_MIPMAP = 1
-} CoglTexturePrePaintFlags;
-
 struct _CoglTextureVtable
 {
   /* Virtual functions that must be implemented for a texture
      backend */
 
-  /* This should update the specified sub region of the texture with a
-     sub region of the given bitmap. The bitmap will have first been
-     converted to a suitable format for uploading if neccessary. */
   gboolean (* set_region) (CoglTexture    *tex,
                            int             src_x,
                            int             src_y,
@@ -78,17 +67,16 @@ struct _CoglTextureVtable
                            int             dst_y,
                            unsigned int    dst_width,
                            unsigned int    dst_height,
-                           CoglBitmap     *bitmap);
+                           int             width,
+                           int             height,
+                           CoglPixelFormat format,
+                           unsigned int    rowstride,
+                           const guint8   *data);
 
-  /* This should copy the image data of the texture into @data. The
-     requested format will have been first passed through
-     _cogl_texture_driver_find_best_gl_get_data_format so it should
-     always be a format that is valid for GL (ie, no conversion should
-     be necessary). */
-  gboolean (* get_data) (CoglTexture     *tex,
-                         CoglPixelFormat  format,
-                         unsigned int     rowstride,
-                         guint8          *data);
+  int (* get_data) (CoglTexture     *tex,
+                    CoglPixelFormat  format,
+                    unsigned int     rowstride,
+                    guint8          *data);
 
   void (* foreach_sub_texture_in_region) (CoglTexture *tex,
                                           float virtual_tx_1,
@@ -118,7 +106,7 @@ struct _CoglTextureVtable
                         GLenum min_filter,
                         GLenum mag_filter);
 
-  void (* pre_paint) (CoglTexture *tex, CoglTexturePrePaintFlags flags);
+  void (* ensure_mipmaps) (CoglTexture *tex);
   void (* ensure_non_quad_rendering) (CoglTexture *tex);
 
   void (* set_wrap_mode_parameters) (CoglTexture *tex,
@@ -130,8 +118,6 @@ struct _CoglTextureVtable
   GLenum (* get_gl_format) (CoglTexture *tex);
   int (* get_width) (CoglTexture *tex);
   int (* get_height) (CoglTexture *tex);
-
-  gboolean (* is_foreign) (CoglTexture *tex);
 };
 
 struct _CoglTexture
@@ -140,37 +126,6 @@ struct _CoglTexture
   const CoglTextureVtable *vtable;
 };
 
-typedef enum _CoglTextureChangeFlags
-{
-  /* Whenever the internals of a texture are changed such that the
-   * underlying GL textures that represent the CoglTexture change then
-   * we notify cogl-material.c via
-   * _cogl_material_texture_pre_change_notify
-   */
-  COGL_TEXTURE_CHANGE_GL_TEXTURES
-
-} CoglTextureChangeFlags;
-
-void
-_cogl_texture_free (CoglTexture *texture);
-
-/* This is used to register a type to the list of handle types that
-   will be considered a texture in cogl_is_texture() */
-void
-_cogl_texture_register_texture_type (GQuark type);
-
-#define COGL_TEXTURE_DEFINE(TypeName, type_name)                        \
-  COGL_HANDLE_DEFINE_WITH_CODE                                          \
-  (TypeName, type_name,                                                 \
-   _cogl_texture_register_texture_type (_cogl_handle_                   \
-                                        ## type_name ## _get_type ()))
-
-#define COGL_TEXTURE_INTERNAL_DEFINE(TypeName, type_name)               \
-  COGL_HANDLE_INTERNAL_DEFINE_WITH_CODE                                 \
-  (TypeName, type_name,                                                 \
-   _cogl_texture_register_texture_type (_cogl_handle_                   \
-                                        ## type_name ## _get_type ()))
-
 void
 _cogl_texture_foreach_sub_texture_in_region (CoglHandle handle,
                                              float virtual_tx_1,
@@ -207,7 +162,7 @@ _cogl_texture_set_filters (CoglHandle handle,
                            GLenum mag_filter);
 
 void
-_cogl_texture_pre_paint (CoglHandle handle, CoglTexturePrePaintFlags flags);
+_cogl_texture_ensure_mipmaps (CoglHandle handle);
 
 void
 _cogl_texture_ensure_non_quad_rendering (CoglHandle handle);
@@ -260,17 +215,4 @@ _cogl_texture_draw_and_read (CoglHandle   handle,
                              GLuint       target_gl_format,
                              GLuint       target_gl_type);
 
-gboolean
-_cogl_texture_is_foreign (CoglHandle handle);
-
-gboolean
-_cogl_texture_set_region_from_bitmap (CoglHandle    handle,
-                                      int           src_x,
-                                      int           src_y,
-                                      int           dst_x,
-                                      int           dst_y,
-                                      unsigned int  dst_width,
-                                      unsigned int  dst_height,
-                                      CoglBitmap   *bmp);
-
 #endif /* __COGL_TEXTURE_PRIVATE_H */
