@@ -405,6 +405,13 @@ clutter_stage_glx_add_redraw_clip (ClutterStageWindow *stage_window,
 			      &stage_glx->bounding_redraw_clip);
     }
 
+  /* FIXME: This threshold was plucked out of thin air! */
+  if (stage_glx->bounding_redraw_clip.height > 300)
+    {
+      /* Set a degenerate clip to force a full redraw */
+      stage_glx->bounding_redraw_clip.width = 0;
+    }
+
 #if 0
   redraw_area = (stage_glx->bounding_redraw_clip.width *
                  stage_glx->bounding_redraw_clip.height);
@@ -461,33 +468,64 @@ drm_wait_vblank(int fd, drm_wait_vblank_t *vbl)
 #endif /* __linux__ */
 
 static void
-wait_for_vblank (ClutterBackendGLX *backend_glx)
+glx_wait_for_vblank (ClutterBackendGLX *backend_glx)
 {
-  if (backend_glx->vblank_type == CLUTTER_VBLANK_NONE)
-    return;
+  /* If we are going to wait for VBLANK manually, we not only need
+   * to flush out pending drawing to the GPU before we sleep, we
+   * need to wait for it to finish. Otherwise, we may end up with
+   * the situation:
+   *
+   *        - We finish drawing      - GPU drawing continues
+   *        - We go to sleep         - GPU drawing continues
+   * VBLANK - We call glXSwapBuffers - GPU drawing continues
+   *                                 - GPU drawing continues
+   *                                 - Swap buffers happens
+   *
+   * Producing a tear. Calling glFinish() first will cause us to properly
+   * wait for the next VBLANK before we swap. This obviously does not
+   * happen when we use GLX_SWAP and let the driver do the right thing
+   */
 
-  if (backend_glx->wait_video_sync)
+  switch (backend_glx->vblank_type)
     {
-      unsigned int retraceCount;
+    case CLUTTER_VBLANK_GLX_SWAP:
+      CLUTTER_NOTE (BACKEND, "Waiting for vblank (swap)");
+      break;
 
-      CLUTTER_NOTE (BACKEND, "Waiting for vblank (wait_video_sync)");
-      backend_glx->get_video_sync (&retraceCount);
-      backend_glx->wait_video_sync (2,
-                                    (retraceCount + 1) % 2,
-                                    &retraceCount);
-    }
-  else
+    case CLUTTER_VBLANK_GLX:
+      {
+        unsigned int retraceCount;
+
+        glFinish ();
+
+        CLUTTER_NOTE (BACKEND, "Waiting for vblank (wait_video_sync)");
+        backend_glx->get_video_sync (&retraceCount);
+        backend_glx->wait_video_sync (2,
+                                      (retraceCount + 1) % 2,
+                                      &retraceCount);
+      }
+      break;
+
+    case CLUTTER_VBLANK_DRI:
 #ifdef __linux__
-    {
-      drm_wait_vblank_t blank;
+      {
+        drm_wait_vblank_t blank;
 
-      CLUTTER_NOTE (BACKEND, "Waiting for vblank (drm)");
-      blank.request.type     = DRM_VBLANK_RELATIVE;
-      blank.request.sequence = 1;
-      blank.request.signal   = 0;
-      drm_wait_vblank (backend_glx->dri_fd, &blank);
-    }
+        glFinish ();
+
+        CLUTTER_NOTE (BACKEND, "Waiting for vblank (drm)");
+        blank.request.type     = DRM_VBLANK_RELATIVE;
+        blank.request.sequence = 1;
+        blank.request.signal   = 0;
+        drm_wait_vblank (backend_glx->dri_fd, &blank);
+      }
 #endif
+      break;
+
+    case CLUTTER_VBLANK_NONE:
+    default:
+      break;
+    }
 }
 
 void
@@ -499,7 +537,6 @@ clutter_stage_glx_redraw (ClutterStageGLX *stage_glx,
   ClutterBackendGLX *backend_glx;
   ClutterStageX11   *stage_x11;
   GLXDrawable        drawable;
-  unsigned int       video_sync_count;
   CLUTTER_STATIC_TIMER (painting_timer,
                         "Redrawing", /* parent */
                         "Painting actors",
@@ -510,10 +547,10 @@ clutter_stage_glx_redraw (ClutterStageGLX *stage_glx,
                         "glXSwapBuffers",
                         "The time spent blocked by glXSwapBuffers",
                         0 /* no application private data */);
-  CLUTTER_STATIC_TIMER (blit_sub_buffer_timer,
+  CLUTTER_STATIC_TIMER (copy_sub_buffer_timer,
                         "Redrawing", /* parent */
-                        "glx_blit_sub_buffer",
-                        "The time spent in _glx_blit_sub_buffer",
+                        "glXCopySubBufferMESA",
+                        "The time spent blocked by glXCopySubBufferMESA",
                         0 /* no application private data */);
 
   backend     = clutter_get_default_backend ();
@@ -524,7 +561,7 @@ clutter_stage_glx_redraw (ClutterStageGLX *stage_glx,
 
   CLUTTER_TIMER_START (_clutter_uprof_context, painting_timer);
 
-  if (backend_glx->can_blit_sub_buffer &&
+  if (backend_glx->copy_sub_buffer &&
       /* NB: a degenerate redraw clip width == full stage redraw */
       (stage_glx->bounding_redraw_clip.width != 0) &&
       G_LIKELY (!(clutter_paint_debug_flags &
@@ -548,19 +585,11 @@ clutter_stage_glx_redraw (ClutterStageGLX *stage_glx,
 
   drawable = stage_glx->glxwin ? stage_glx->glxwin : stage_x11->xwin;
 
-  /* If we might ever use _clutter_backend_glx_blit_sub_buffer then we
-   * always need to keep track of the video_sync_count so that we can
-   * throttle blits.
-   *
-   * Note: we get the count *before* we issue any glXCopySubBuffer or
-   * blit_sub_buffer request in case the count would go up before
-   * returning control to us.
-   */
-  if (backend_glx->can_blit_sub_buffer && backend_glx->get_video_sync)
-    backend_glx->get_video_sync (&video_sync_count);
+  /* wait for the next vblank */
+  glx_wait_for_vblank (CLUTTER_BACKEND_GLX (backend));
 
   /* push on the screen */
-  if (backend_glx->can_blit_sub_buffer &&
+  if (backend_glx->copy_sub_buffer &&
       /* NB: a degenerate redraw clip width == full stage redraw */
       (stage_glx->bounding_redraw_clip.width != 0) &&
       G_LIKELY (!(clutter_paint_debug_flags &
@@ -570,9 +599,11 @@ clutter_stage_glx_redraw (ClutterStageGLX *stage_glx,
       ClutterGeometry copy_area;
 
       CLUTTER_NOTE (BACKEND,
-                    "_glx_blit_sub_buffer (window: 0x%lx, "
+                    "glXCopySubBufferMESA (display: %p, "
+                                          "window: 0x%lx, "
                                           "x: %d, y: %d, "
                                           "width: %d, height: %d)",
+                    backend_x11->xdpy,
                     (unsigned long) drawable,
                     stage_glx->bounding_redraw_clip.x,
                     stage_glx->bounding_redraw_clip.y,
@@ -629,51 +660,14 @@ clutter_stage_glx_redraw (ClutterStageGLX *stage_glx,
       copy_area.width = clip->width;
       copy_area.height = clip->height;
 
-      /* glXCopySubBufferMESA and glBlitFramebuffer are not integrated
-       * with the glXSwapIntervalSGI mechanism which we usually use to
-       * throttle the Clutter framerate to the vertical refresh and so
-       * we have to manually wait for the vblank period...
-       */
-
-      /* Here 'is_synchronized' only means that the blit won't cause a
-       * tear, ie it won't prevent multiple blits per retrace if they
-       * can all be performed in the blanking period. If that's the
-       * case then we still want to use the vblank sync menchanism but
-       * we only need it to throttle redraws.
-       */
-      if (!backend_glx->blit_sub_buffer_is_synchronized)
-        {
-          /* XXX: note that glXCopySubBuffer, at least for Intel, is
-           * synchronized with the vblank but glBlitFramebuffer may
-           * not be so we use the same scheme we do when calling
-           * glXSwapBuffers without the swap_control extension and
-           * call glFinish () before waiting for the vblank period.
-           *
-           * See where we call glXSwapBuffers for more details.
-           */
-          glFinish ();
-          wait_for_vblank (CLUTTER_BACKEND_GLX (backend));
-        }
-      else if (backend_glx->get_video_sync)
-        {
-          /* If we have the GLX_SGI_video_sync extension then we can
-           * be a bit smarter about how we throttle blits by avoiding
-           * any waits if we can see that the video sync count has
-           * already progressed. */
-          if (backend_glx->last_video_sync_count == video_sync_count)
-            wait_for_vblank (CLUTTER_BACKEND_GLX (backend));
-        }
-      else
-        wait_for_vblank (CLUTTER_BACKEND_GLX (backend));
-
-      CLUTTER_TIMER_START (_clutter_uprof_context, blit_sub_buffer_timer);
-      _clutter_backend_glx_blit_sub_buffer (backend_glx,
-                                            drawable,
-                                            copy_area.x,
-                                            copy_area.y,
-                                            copy_area.width,
-                                            copy_area.height);
-      CLUTTER_TIMER_STOP (_clutter_uprof_context, blit_sub_buffer_timer);
+      CLUTTER_TIMER_START (_clutter_uprof_context, copy_sub_buffer_timer);
+      backend_glx->copy_sub_buffer (backend_x11->xdpy,
+                                    drawable,
+                                    copy_area.x,
+                                    copy_area.y,
+                                    copy_area.width,
+                                    copy_area.height);
+      CLUTTER_TIMER_STOP (_clutter_uprof_context, copy_sub_buffer_timer);
     }
   else
     {
@@ -687,37 +681,11 @@ clutter_stage_glx_redraw (ClutterStageGLX *stage_glx,
       if (clutter_feature_available (CLUTTER_FEATURE_SWAP_EVENTS))
         stage_glx->pending_swaps++;
 
-      if (backend_glx->vblank_type != CLUTTER_VBLANK_GLX_SWAP &&
-          backend_glx->vblank_type != CLUTTER_VBLANK_NONE)
-        {
-          /* If we are going to wait for VBLANK manually, we not only
-           * need to flush out pending drawing to the GPU before we
-           * sleep, we need to wait for it to finish. Otherwise, we
-           * may end up with the situation:
-           *
-           *        - We finish drawing      - GPU drawing continues
-           *        - We go to sleep         - GPU drawing continues
-           * VBLANK - We call glXSwapBuffers - GPU drawing continues
-           *                                 - GPU drawing continues
-           *                                 - Swap buffers happens
-           *
-           * Producing a tear. Calling glFinish() first will cause us
-           * to properly wait for the next VBLANK before we swap. This
-           * obviously does not happen when we use _GLX_SWAP and let
-           * the driver do the right thing
-           */
-          glFinish ();
-
-          wait_for_vblank (CLUTTER_BACKEND_GLX (backend));
-        }
-
       CLUTTER_TIMER_START (_clutter_uprof_context, swapbuffers_timer);
       glXSwapBuffers (backend_x11->xdpy, drawable);
       CLUTTER_TIMER_STOP (_clutter_uprof_context, swapbuffers_timer);
     }
 
-  backend_glx->last_video_sync_count = video_sync_count;
-
   /* reset the redraw clipping for the next paint... */
   stage_glx->initialized_redraw_clip = FALSE;
 }
