@@ -32,22 +32,10 @@ on_drag_begin (ClutterDragAction   *action,
 
   clutter_drag_action_set_drag_handle (action, drag_handle);
 
-  clutter_actor_set_opacity (actor, 128);
-}
-
-static void
-on_drag_motion (ClutterDragAction   *action,
-                ClutterActor        *actor,
-                gfloat               delta_x,
-                gfloat               delta_y,
-                ClutterModifierType  modifiers)
-{
-  ClutterActor *drag_handle;
-
-  drag_handle = clutter_drag_action_get_drag_handle (action);
-  g_assert (drag_handle != NULL);
-
-  clutter_actor_move_by (drag_handle, delta_x, delta_y);
+  /* fully desaturate the actor */
+  clutter_actor_animate (actor, CLUTTER_LINEAR, 150,
+                         "@effects.disable.factor", 1.0,
+                         NULL);
 }
 
 static void
@@ -83,14 +71,14 @@ on_drag_end (ClutterDragAction   *action,
                                            &real_y);
 
       clutter_actor_animate (actor, CLUTTER_EASE_OUT_CUBIC, 150,
-                             "opacity", 255,
+                             "@effects.disable.factor", 0.0,
                              "x", real_x,
                              "y", real_y,
                              NULL);
     }
   else
     clutter_actor_animate (actor, CLUTTER_LINEAR, 150,
-                           "opacity", 255,
+                           "@effects.disable.factor", 0.0,
                            NULL);
 }
 
@@ -176,11 +164,12 @@ test_drag_main (int argc, char *argv[])
                                      get_drag_axis (drag_axis));
 
   g_signal_connect (action, "drag-begin", G_CALLBACK (on_drag_begin), NULL);
-  g_signal_connect (action, "drag-motion", G_CALLBACK (on_drag_motion), NULL);
   g_signal_connect (action, "drag-end", G_CALLBACK (on_drag_end), NULL);
 
   clutter_actor_add_action (handle, action);
 
+  clutter_actor_add_effect_with_name (handle, "disable", clutter_desaturate_effect_new (0.0));
+
   clutter_actor_show (stage);
 
   clutter_main ();
